﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using System.Windows.Forms;
using System.Diagnostics;
using System.Net;
using System.IO;
using SSDSNwindEntitiesCS.SitkaClient;
using System.ServiceModel;
using System.Web;


namespace SSDSNwindEntitiesCS
{
    public partial class MainForm : Form
    {
        private static string userName = null;
        private static string password = null;
        private static string authorityId = null;
        private static string containerId = null;
        private static long contentBytes = 0;
        private static long totalBytes = 0;
        private static bool bail = false;
        private List<String> dataList = new List<String>();
        private List<Entity> brazilOrders = new List<Entity>();
        private string brazilOrdersXml = null;
        // Changed by Sprint 3
        // private const string XmlContentType = "application/xml";
        private const string XmlContentType = "application/x-ssds+xml";
        private const string HttpGetMethod = "GET";
        private const string HttpPostMethod = "POST";
        private const string HttpPutMethod = "PUT";
        private const string HttpDeleteMethod = "DELETE";
        private const string title = "SSDS Northwind Entities Test Harness";
        
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            Form startup = new Form();            
            ShowSplashscreen(startup);
            // Get credentials and Id from text boxes
            userName = txtUser.Text;
            password = txtPassword.Text;
            authorityId = txtAuthority.Text;
            containerId = txtContainer.Text;

            using (SitkaSoapServiceClient proxy = new SitkaSoapServiceClient())
            {
                this.Cursor = Cursors.WaitCursor;
                // Supply credentials
                proxy.ClientCredentials.UserName.UserName = userName;
                proxy.ClientCredentials.UserName.Password = password;

                // Supply Authority scope to query for Containers
                Scope authorityScope = new Scope();
                authorityScope.AuthorityId = authorityId;


                // Execute a SOAP query
                List<Entity> containers = null;
                try
                {
                    containers = proxy.Query(authorityScope, "from e in entities select e");

                    // Set true to add entities to the list (very time-consuming, 
                    // especially with base64Binary encoding of photos)
                    bool listEntitySets = true;
                    if (listEntitySets)
                        txtPayload.Text = "Following are containers and entities in the '" + authorityId + "' authority:\r\n";
                    else
                        txtPayload.Text = "Following are containers in the '" + authorityId + "' authority:\r\n";
                    Application.DoEvents();
                    foreach (Entity container in containers)
                    {
                        txtPayload.Text += container.Id + "\r\n";
                        if (listEntitySets && !chkContainerPerEntity.Checked)
                        {
                            Scope containerScope = new Scope();
                            containerScope.AuthorityId = authorityId;
                            containerScope.ContainerId = container.Id;
                            string[] entities = { "Customer", "Order", "Employee", "Shipper", "OrderDetail", "Product", "Category", "Supplier" };
                            for (int i = 0; i < 8; i++)
                            {
                                string queryString = "from k in entities where k.Kind == \"" + entities[i] + "\" select k";
                                var query = proxy.Query(containerScope, queryString).AsQueryable();
                                var kinds = query.Select(d => d.Kind).Distinct();
                                foreach (string kind in kinds)
                                {
                                    txtPayload.Text += "   " + kind + "\r\n";
                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    startup.Hide();
                    string msg = "SSDS rejected your UserName, Password, and/or AuthorityID. \r\n\r\nMessage: " +
                        ex.Message;
                    if (ex.Message.Contains("Authority "))
                    {
                        msg += "\r\n\r\nDo you want to create the '" + authorityId + "' Authority?";
                        if (MessageBox.Show(msg, title, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                        {
                            Scope serviceScope = new Scope();
                            Authority authority = new Authority();
                            authority.Id = authorityId;
                            proxy.Create(serviceScope, authority);
                        }
                    }
                    else
                        MessageBox.Show(msg, title);
                }
                finally
                {
                    startup.Hide();
                    this.Cursor = Cursors.Default;
                    this.Show();
                }
            }
        }

        // Create entities and display content while creating
        private void btnCreateLoad_Click(object sender, EventArgs e)
        {
            // Reset bail-out flag
            bail = false;

            // Disable changing ContainerPerEntity checkbox after creating containers
            chkContainerPerEntity.Enabled = false;

            this.Cursor = Cursors.WaitCursor;
            // Update destination from text boxes
            userName = txtUser.Text;
            password = txtPassword.Text;
            authorityId = txtAuthority.Text;
            string authorityUri = String.Format("https://{0}.data.beta.mssds.com/v1/",
                                                authorityId);
            if (chkContainerPerEntity.Checked)
                txtContainer.Text = "Customers";
            else
                txtContainer.Text = "Northwind";
            containerId = txtContainer.Text;
            string containerUri = CreateContainer(authorityUri);
            if (containerUri == null)
            {
                string msg = "Unable to create container with authorityUri = '" +
                    authorityUri + "'.\r\n\r\nPlease try again or exit program.";
                MessageBox.Show(msg, title);
                this.Cursor = Cursors.Default;
                return;
            }
            else if (containerUri == "deleted")
            {
                this.Cursor = Cursors.Default;
                return;
            }

            Stopwatch batchTimer = new Stopwatch();
            batchTimer.Start();
            if (chkCustomers.Checked && !bail)
            {
                txtEntitySet.Text = "Customers";
                txtEntities.Text = "";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Customers", "Customer", "Cust");
                chkCustomers.Checked = false;
            }
            if (chkOrders.Checked && !bail)
            {
                if (chkContainerPerEntity.Checked)
                {
                    txtContainer.Text = "Orders";
                    containerUri = CreateContainer(authorityUri);
                }
                txtEntitySet.Text = "Orders";
                txtEntities.Text = "";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Orders", "Order", "Ord");
                chkOrders.Checked = false;
            }
            if (chkEmployees.Checked && !bail)
            {
                if (chkContainerPerEntity.Checked)
                {
                    txtContainer.Text = "Employees";
                    containerUri = CreateContainer(authorityUri);
                }
                txtEntitySet.Text = "Employees";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Employees", "Employee", "Empl");
                chkEmployees.Checked = false;
            }
            if (chkShippers.Checked && !bail)
            {
                if (chkContainerPerEntity.Checked)
                {
                    txtContainer.Text = "Shippers";
                    containerUri = CreateContainer(authorityUri);
                }
                txtEntitySet.Text = "Shippers";
                txtEntities.Text = "";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Shippers", "Shipper", "Ship");
                chkShippers.Checked = false;
            }
            if (chkOrderDetails.Checked && !bail)
            {
                if (chkContainerPerEntity.Checked)
                {
                    txtContainer.Text = "OrderDetails";
                    containerUri = CreateContainer(authorityUri);
                }
                txtEntitySet.Text = "OrderDetails";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Order Details", "OrderDetail", "Dtl");
                chkOrderDetails.Checked = false;
            }
            if (chkProducts.Checked && !bail)
            {
                if (chkContainerPerEntity.Checked)
                {
                    txtContainer.Text = "Products";
                    containerUri = CreateContainer(authorityUri);
                }
                txtEntitySet.Text = "Products";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Products", "Product", "Prod");
                chkProducts.Checked = false;
            }
            if (chkCategories.Checked && !bail)
            {
                if (chkContainerPerEntity.Checked)
                {
                    txtContainer.Text = "Categories";
                    containerUri = CreateContainer(authorityUri);
                }
                txtEntitySet.Text = "Categories";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Categories", "Category", "Cat");
                chkCategories.Checked = false;
            }
            if (chkSuppliers.Checked && !bail)
            {
                if (chkContainerPerEntity.Checked)
                {
                    txtContainer.Text = "Suppliers";
                    containerUri = CreateContainer(authorityUri);
                }
                txtEntitySet.Text = "Suppliers";
                Application.DoEvents();
                CreateEntitySet(authorityUri, containerUri, "Suppliers", "Supplier", "Vend");
                chkSuppliers.Checked = false;
            }
            bail = false;
            this.Cursor = Cursors.Default;
            txtPayload.Text += "\r\nTotal elapsed time: " + (batchTimer.ElapsedMilliseconds / 1000D).ToString("0.0") + " seconds";
        }

        private void CreateEntitySet(string authorityUri, string containerUri, string tableName, string entityType, string idPrefix)
        {
            int entities = 0;
            txtEntities.Text = "0";
            contentBytes = 0;
            txtBytes.Text = "0";
            txtPayload.Text = "";
            Application.DoEvents();
            Stopwatch swXml = new Stopwatch();
            swXml.Start();
            Stopwatch swUpload = new Stopwatch();
            XElement setElement = new XElement(tableName.Replace(" ", ""));
            string strNwind = Properties.Settings.Default.NorthwindConnectionString;

            if (chkUseREST.Checked)
            {
                string setXml = null;
                using (SqlConnection connNwind = new SqlConnection(strNwind))
                {
                    using (SqlCommand cmdNwind = new SqlCommand("SELECT * FROM [" + tableName + "]", connNwind))
                    {
                        connNwind.Open();
                        XNamespace xsi = "http://www.w3.org/2001/XMLSchema-instance";
                        SqlDataReader sdrNwind = cmdNwind.ExecuteReader(CommandBehavior.SequentialAccess);
                        while (sdrNwind.Read())
                        {
                            var entElement = new XElement(new XElement(entityType,
                            new XAttribute(XNamespace.Xmlns + "s", "http://schemas.microsoft.com/sitka/2008/03/"),
                            new XAttribute(XNamespace.Xmlns + "xsi", "http://www.w3.org/2001/XMLSchema-instance"),
                            new XAttribute(XNamespace.Xmlns + "x", "http://www.w3.org/2001/XMLSchema"),
                                from col in Enumerable.Range(0, sdrNwind.FieldCount)
                                select new XElement(sdrNwind.GetName(col),
                                    new XAttribute(xsi + "type", "x:" + sdrNwind.GetFieldType(col).ToString().ToLower()),
                                    sdrNwind.GetValue(col))));
                            setElement.Add(entElement);
                            if (bail)
                                break;
                            Application.DoEvents();
                        }
                        sdrNwind.Close();
                        setXml = setElement.ToString();
                        // String identifier change from " to '
                        setXml = setXml.Replace('\"', '\'');
                        // Type fixups with “system.” prefix for specificity
                        setXml = setXml.Replace("x:system.uint64", "x:decimal");
                        setXml = setXml.Replace("x:system.uint32", "x:decimal");
                        setXml = setXml.Replace("x:system.uint16", "x:decimal");
                        setXml = setXml.Replace("x:system.int64", "x:decimal");
                        setXml = setXml.Replace("x:system.int32", "x:decimal");
                        setXml = setXml.Replace("x:system.int16", "x:decimal");
                        setXml = setXml.Replace("x:system.byte[]", "x:base64Binary");
                        setXml = setXml.Replace("x:system.byte", "x:decimal");
                        setXml = setXml.Replace("x:system.sbyte", "x:decimal");
                        setXml = setXml.Replace("x:system.single", "x:decimal");
                        setXml = setXml.Replace("x:system.double", "x:decimal");
                        setXml = setXml.Replace("x:system.decimal", "x:decimal");
                        setXml = setXml.Replace("x:system.datetime", "x:dateTime");
                        setXml = setXml.Replace("x:system.boolean", "x:boolean");
                        setXml = setXml.Replace("x:system.char", "x:string");
                        setXml = setXml.Replace("x:system.guid", "x:string");
                        setXml = setXml.Replace("x:system.string", "x:string");
                        // Special case: Temporarily remove value elements with nil values
                        setXml = setXml.Replace("<DateDiscontinued xsi:type='x:dateTime'></DateDiscontinued>", "");
                        setXml = setXml.Replace("<ShippedDate xsi:type='x:decimal'></ShippedDate>", "");
                        // For debugging
                        // txtPayload.Text = setXml;
                        // Application.DoEvents();
                    }
                }
                setElement = XElement.Parse(setXml); //.Load(path);
                IEnumerable<XElement> entList = setElement.Descendants(entityType);
                entities = 0;
                foreach (XElement entity in entList)
                {
                    swXml.Start();
                    entities += 1;
                    // Add unique Id element if missing
                    string test = entity.Elements().First().Name.ToString();
                    XNamespace s = "http://schemas.microsoft.com/sitka/2008/03/";
                    XNamespace xsi = "http://www.w3.org/2001/XMLSchema-instance";
                    XNamespace x = "http://www.w3.org/2001/XMLSchema";
                    if (entity.Elements().First().Name.ToString() != s + "Id")
                    {
                        XNode nodeId = entity.Nodes().First();
                        XElement Id = null;
                        if (chkGuids.Checked)
                        {
                            // Not idempotent, can add duplicate entities
                            Id = new XElement(s + "Id", idPrefix + "_" + Guid.NewGuid().ToString());
                        }
                        else
                        {
                            //Entities is 0-based
                            Id = new XElement(s + "Id", idPrefix + "_" + (entities + 1000000).ToString());
                        }
                        // nodeId.AddBeforeSelf(Id);
                        entity.AddFirst(Id);
                    }

                    // List of elements to be removed
                    List<string> removeList = new List<string>();
                    foreach (XElement prop in entity.Descendants())
                    {
                        int count = prop.Attributes().Count();
                        if (prop.Attributes().Count() > 0)
                        {
                            string name1 = prop.Name.ToString();
                            string value1 = prop.Attributes().First().Value.ToString();
                            // Set false to remove any base64Binary (byte[]) properties
                            if (prop.Attributes().First().Value.ToString() == "x:base64Binary")
                            {
                                if (chkEncodeImages.Checked == true)
                                {
                                    byte[] byteArray = new byte[prop.Value.Length];
                                    char[] charArray = prop.Value.ToString().ToCharArray();
                                    for (int i = 0; i < charArray.Length; i++)
                                        byteArray[i] = (byte)charArray[i];
                                    prop.SetValue(Convert.ToBase64String(byteArray, Base64FormattingOptions.None));
                                }
                                else
                                    removeList.Add(prop.Name.ToString());
                            }
                            else
                            {
                                // Temporary: Remove any value type elements with null values
                                if (prop.Attributes().First().Value.ToString() != "x:string")
                                {
                                    string value = prop.Value.ToString();
                                    if (prop.Value.ToString() == "")
                                        removeList.Add(prop.Name.ToString());
                                }
                            }
                        }
                    }
                    foreach (string remove in removeList)
                    {
                        entity.Element(remove).Remove();
                    }
                    swXml.Stop();
                    StringBuilder payload = new StringBuilder();
                    payload.Append(entity.ToString());
                    contentBytes += payload.ToString().Length;
                    totalBytes = +contentBytes;
                    txtPayload.Text = payload.ToString();
                    // Save a file copy for debugging
                    string path = Application.StartupPath + @"\" + entityType + "Template.xml";
                    StreamWriter sw = File.CreateText(path);
                    sw.Write(txtPayload.Text);
                    sw.Flush();
                    sw.Close();
                    // Display in IE for debugging
                    // Process.Start(path);
                    swUpload.Start();
                    // Use CreatEntity sample application to process the request
                    if (ProcessEntity(containerUri, entity.ToString(SaveOptions.DisableFormatting), "POST") == "exit")
                        break;
                    swUpload.Stop();
                    if (bail)
                        break;
                    txtTimeXml.Text = (swXml.ElapsedMilliseconds / 1000d).ToString("#0.000");
                    txtTimeUpload.Text = (swUpload.ElapsedMilliseconds / 1000d).ToString("#0.000");
                    txtBytes.Text = contentBytes.ToString();
                    txtEntities.Text = entities.ToString();
                    Application.DoEvents();
                }
            }
            else            
            {
                // Add new entities with SOAP protocol
                using (SitkaSoapServiceClient proxy = new SitkaSoapServiceClient())
                {
                    // Supply credentials
                    proxy.ClientCredentials.UserName.UserName = userName;
                    proxy.ClientCredentials.UserName.Password = password;

                    // Set scope to the container where sample entity is 
                    // added (for later deletion)
                    Scope containerScope = new Scope();
                    containerScope.AuthorityId = authorityId;
                    containerScope.ContainerId = containerId;

                    using (SqlConnection connNwind = new SqlConnection(strNwind))
                    {
                        using (SqlCommand cmdNwind = new SqlCommand("SELECT * FROM [" + tableName + "]", connNwind))
                        {
                            connNwind.Open();
                            SqlDataReader sdrNwind = cmdNwind.ExecuteReader();

                            Entity ent = null;
                            StringBuilder payload = new StringBuilder();
                            while (sdrNwind.Read())
                            {
                                swXml.Start();
                                ent = new Entity();
                                // Entities is 0-based
                                if (chkGuids.Checked)
                                {
                                    ent.Id = idPrefix + "_" + Guid.NewGuid();
                                }
                                else
                                {
                                    ent.Id = idPrefix + "_" + (entities + 1000001).ToString();
                                }
                                ent.Kind = entityType;
                                payload.Length = 0;
                                payload.Append("Kind = " + ent.Kind + "\r\n");
                                payload.Append("Id = " + ent.Id + "\r\n");
                                ent.Properties = new Dictionary<string, object>();
                                for (int i = 0; i < sdrNwind.FieldCount; i++)
                                {
                                    if (sdrNwind.GetValue(i) != DBNull.Value &&
                                        (chkEncodeImages.Checked == true || 
                                         sdrNwind.GetFieldType(i).ToString() != "System.Byte[]"))
                                    {
                                        //string fieldType = sdrNwind.GetFieldType(i).ToString();
                                        // ToString() appended to prevent untrapped error on Int32
                                        string key = sdrNwind.GetName(i);
                                        object value = sdrNwind.GetValue(i);
                                        string type = value.GetType().ToString();
                                        object objValue = null;
                                        if (chkEncodeImages.Checked == true && type == "System.Byte[]")
                                        {
                                            // base64Encode images
                                            byte[] byteArray = new Byte[(sdrNwind.GetBytes(i, 0, null, 0, int.MaxValue))];
                                            sdrNwind.GetBytes(i, 0, byteArray, 0, byteArray.Length);
                                            objValue = Convert.ToBase64String(byteArray, Base64FormattingOptions.None);
                                        }
                                        // Change numeric CLR types to Decimal
                                        else if (type.Contains("Int") ||
                                            type.Contains("Single") ||
                                            type.Contains("Double") ||
                                            type.Contains("Byte"))
                                            {
                                                decimal decValue = Decimal.Parse(value.ToString());
                                                objValue = (object)decValue;
                                                type = objValue.GetType().ToString();
                                            }
                                        else
                                            objValue = value;
                                        payload.Append(key + " = " + objValue + "\r\n");
                                        ent.Properties.Add(key, objValue);
                                    }
                                }
                                swXml.Stop();
                                txtPayload.Text = payload.ToString();
                                swUpload.Start();
                                try
                                {
                                    proxy.Create(containerScope, ent);
                                }
                                catch (FaultException<Error> e)
                                {
                                    // Suppress EntityExists errors in case user and/or authority already exists
                                    if (e.Detail.StatusCode != ErrorCodes.EntityExists)
                                    {
                                        string msg = String.Format("Error: {0}:{1}", e.Detail.StatusCode, e.Detail.Message) +
                                            "\r\n\r\nDo you want to continue?";
                                        if (MessageBox.Show(msg, title, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                                            break;
                                    }
                                }
                                catch (Exception ex)
                                {
                                    string msg = String.Format("Error: {0}:{1}", ex.Message, ex.InnerException) +
                                        "\r\n\r\nDo you want to continue?";
                                    if (MessageBox.Show(msg, title, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                                        break;
                                }
                                entities += 1;
                                txtTimeXml.Text = (swXml.ElapsedMilliseconds / 1000d).ToString("#0.000");
                                txtTimeUpload.Text = (swUpload.ElapsedMilliseconds / 1000d).ToString("#0.000");
                                txtBytes.Text = contentBytes.ToString();
                                txtEntities.Text = entities.ToString();
                                if (bail)
                                    break;
                                Application.DoEvents();
                            }
                        }
                    }
                }
            }
            txtTimeXml.Text = (swXml.ElapsedMilliseconds / 1000d).ToString("#0.000"); 
            txtTimeUpload.Text = (swUpload.ElapsedMilliseconds / 1000d).ToString("#0.000");
            txtBytes.Text = contentBytes.ToString();
            Application.DoEvents();
            txtEntities.Text = entities.ToString();

            // Add a line to dataList
            AddTimingData();
        }
                
        // Save timing data for CSV file created on closing form
        private void AddTimingData()
        {
            if (dataList.Count() == 0)
            {
                //Add header
                dataList.Add(@"""DateTime"",""ContainerId"",""Protocol"",""Base64Images"",""EntitySet"",""Entities"",""Bytes"",""Prepare, s."",""Transfer, s."",""Total Time, s."",""Entities/s.""");
            }
            string protocol = null;
            if (chkUseREST.Checked)
                protocol = "REST";
            else
                protocol = "SOAP";
            decimal totalTime = decimal.Parse(txtTimeXml.Text) + decimal.Parse(txtTimeUpload.Text);
            dataList.Add("\"" + DateTime.Now + "\",\"" + containerId + "\",\"" + protocol + "\",\"" + chkEncodeImages.Checked.ToString() + "\",\"" + txtEntitySet.Text + 
                 "\"," + txtEntities.Text + "," + txtBytes.Text + "," + txtTimeXml.Text + "," + txtTimeUpload.Text + "," + totalTime.ToString() + "," + (Int32.Parse(txtEntities.Text)/totalTime).ToString("0.00"));
        }

        // Write timing data to file when closing form
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            string filePath = Application.StartupPath + @"\TimingData" + DateTime.Now.ToString("s").Replace(":", "-") + ".csv";
            File.WriteAllLines(filePath, dataList.ToArray(), Encoding.ASCII);
        }

        private void btnBail_Click(object sender, EventArgs e)
        {
            // Stop creating new entities
            bail = true;
        }

        // ****************************************************************************
        // Get, update, delete and, optionally, recreate Order entities shipped to Brazil
        // ****************************************************************************
        
        // Get a set of orders for updating and deletion (orders shipped to Brazil)
        private void btnGetOrders_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            if (chkContainerPerEntity.Checked)
                txtContainer.Text = "Orders";
            else
                txtContainer.Text = "Northwind";
            containerId = txtContainer.Text; 
            if (chkUseREST.Checked)
            {
                // Get EntitySet of Brazilian orders with the REST protocol 
                // as a formatted XML document from a LINQ-like query string
                Stopwatch sw = new Stopwatch();
                string containerUri = String.Format("https://{0}.data.beta.mssds.com/v1/{1}", 
                    authorityId, containerId);
                string brazilQuery = @"from e in entities where e [""ShipCountry""] == ""Brazil"" select e";
                // Test of compound where clause
                brazilQuery = @"from e in entities where e [""ShipCountry""] == ""Brazil"" && e.Kind == ""Order"" select e";
                // brazilQuery = brazilQuery.Replace("&", "%26");
                // string entityUri = containerUri + "?q='" + brazilQuery.Replace(" ", "%20") + "'";
                // Test of HttpUtility.UrlEncode() method (requires System.Web reference and using)
                string entityUri = containerUri + "?q='" + HttpUtility.UrlEncode(brazilQuery) + "'";
                string data = null;
                try
                {
                    // Retrieve Brazilian orders
                    WebRequest request = HttpWebRequest.Create(entityUri);
                    request.Method = "GET";
                    request.ContentLength = 0;
                    request.ContentType = "application/xml";
                    request.Credentials = new NetworkCredential(userName, password);
                    sw.Start();                                       
                    
                    using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                    {
                        // ReadResponse() function code incorporated for clarity
                        string responseBody = "";
                        using (Stream rspStm = response.GetResponseStream())
                        {
                            using (StreamReader reader = new StreamReader(rspStm))
                            {
                                responseBody = reader.ReadToEnd();
                            }
                        }
                        data = responseBody;

                        if (response.StatusCode != HttpStatusCode.OK)
                        {
                            MessageBox.Show(string.Format("Unexpected status code returned (HttpWebResponse): {0}", response.StatusCode), title);
                            return;
                        }
                    }
                }
                catch (WebException ex)
                {
                    using (HttpWebResponse response = ex.Response as HttpWebResponse)
                    {
                        MessageBox.Show(string.Format("Unexpected status code returned (WebException): {0}", response.StatusCode), title);
                        return;
                    }
                }
                sw.Stop();
                double time = (sw.ElapsedMilliseconds / 1000d);
                brazilOrdersXml = FormatXml(data);
                XmlDocument ordersXml = new XmlDocument();
                ordersXml.LoadXml(brazilOrdersXml);
                XmlNodeList ordersElements = ordersXml.GetElementsByTagName("Order");
                if (ordersElements.Count > 0)
                {
                    btnUpdateOrders.Enabled = true;
                    // REST doesn't save the original order values
                    if (!chkUseREST.Checked)
                        btnDeleteOrders.Enabled = true;
                    txtPayload.Text = ordersElements.Count.ToString() +
                        " Brazilian orders returned in " +
                        time.ToString("0.000") + " sec. = " +
                        (ordersElements.Count / time).ToString("0.000") +
                        " orders/sec.\r\n\r\n";
                    txtPayload.Text += brazilOrdersXml;
                    txtEntitySet.Text = "Get Orders";
                    txtEntities.Text = ordersElements.Count.ToString();
                    txtBytes.Text = brazilOrdersXml.Length.ToString();
                    txtTimeUpload.Text = time.ToString("0.000");
                    txtTimeXml.Text = "0.000";
                    AddTimingData();
                }
            }
            else
            {
                // Get Entities for Brazilian orders with the SOAP protocol
                using (SitkaSoapServiceClient proxy = new SitkaSoapServiceClient())
                {
                    // Supply credentials
                    proxy.ClientCredentials.UserName.UserName = userName;
                    proxy.ClientCredentials.UserName.Password = password;

                    Scope authorityScope = new Scope();
                    authorityScope.AuthorityId = authorityId;

                    // Set scope to the container where sample entity is 
                    // added (for later deletion)
                    Scope containerScope = new Scope();
                    containerScope.AuthorityId = authorityId;
                    containerScope.ContainerId = containerId;
                    Stopwatch sw = new Stopwatch();
                    sw.Start();

                    string brazilQuery = @"from e in entities where e [""ShipCountry""] == ""Brazil"" && e.Kind == ""Order"" select e";
                    brazilOrders = proxy.Query(containerScope, brazilQuery);

                    double time = (sw.ElapsedMilliseconds / 1000d);
                    sw.Stop();
                    StringBuilder payload = new StringBuilder();
                    payload.Append(brazilOrders.Count().ToString() +
                        " Brazilian orders returned in " +
                        time.ToString("0.000") + " sec. = " +
                        (brazilOrders.Count() / time).ToString("#0.000") +
                        " orders/sec.\r\n\r\n");
                    foreach (Entity order in brazilOrders)
                    {
                        payload.Append("Id = " + order.Id + " (String) \r\n");
                        payload.Append("Kind = " + order.Kind + " (String) \r\n");
                        payload.Append("Version = " + order.Version + " (String) \r\n");
                        // Flex properties
                        foreach (KeyValuePair<string, object> kvp in order.Properties)
                            payload.Append(kvp.Key + " = " + kvp.Value.ToString() + 
                                " (" + kvp.Value.GetType().ToString().Replace("System.", "") + ")\r\n");
                        payload.Append("\r\n");
                    }
                    txtPayload.Text = payload.ToString();
                    if (brazilOrders.Count() > 0)
                    {
                        entityBindingSource.DataSource = brazilOrders;
                        entityBindingSource.ResetBindings(true);

                        btnUpdateOrders.Enabled = true;
                        btnDeleteOrders.Enabled = true;
                    }
                    txtEntitySet.Text = "Get Orders";
                    txtEntities.Text = brazilOrders.Count.ToString();
                    txtBytes.Text = "0";
                    txtTimeUpload.Text = time.ToString("0.000");
                    txtTimeXml.Text = "0.000";
                    AddTimingData();
                }
            }
            this.Cursor = Cursors.Default;
        }

        // From "Formatting XML - A Code snippet" by Ian Randell
        // http://www.codeproject.com/KB/cpp/FormattingXML.aspx
        private string FormatXml(string sUnformattedXml)
        {            
            //load unformatted xml into a dom
            XmlDocument xd = new XmlDocument();
            xd.LoadXml(sUnformattedXml);

            //will hold formatted xml
            StringBuilder sb = new StringBuilder();

            //pumps the formatted xml into the StringBuilder above
            StringWriter sw = new StringWriter(sb);

            //does the formatting
            XmlTextWriter xtw = null;

            try
            {
                //point the xtw at the StringWriter
                xtw = new XmlTextWriter(sw);

                //we want the output formatted
                xtw.Formatting = Formatting.Indented;
                xtw.Indentation = 2;

                //get the dom to dump its contents into the xtw 
                xd.WriteTo(xtw);
            }
            finally
            {
                //clean up even if error
                if (xtw != null)
                    xtw.Close();
            }

            //return the formatted xml
            return sb.ToString();
        }
        
        // Update Brazilian orders by adding or removing a TimeStamp flex property
        private void btnUpdateOrders_Click(object sender, EventArgs e)
        {
            UpdateOrAddBrazilianOrders(true);
        }

        private void UpdateOrAddBrazilianOrders(bool updateOrders)
        {
            int entities = 0;
            string action = null;
            Stopwatch sw = new Stopwatch();
            this.Cursor = Cursors.WaitCursor;
            if (chkUseREST.Checked)
            {
                txtPayload.Text = "";
                Application.DoEvents();
                sw.Start();
                XmlDocument ordersXml = new XmlDocument();
                ordersXml.LoadXml(brazilOrdersXml);
                XmlNodeList ordersElements = ordersXml.GetElementsByTagName("Order");
                foreach (XmlElement orderElement in ordersElements)
                {
                    // Get the Id of each Entity
                    string containerUri = String.Format("https://{0}.data.beta.mssds.com/v1/{1}", authorityId, containerId);
                    string entityUri = containerUri + "/" + orderElement.FirstChild.InnerXml;

                    string data = null;
                    try
                    {
                        // Retrieve each Brazilian order by Id
                        WebRequest request = HttpWebRequest.Create(entityUri);
                        request.Method = HttpGetMethod;
                        request.ContentLength = 0;
                        request.ContentType = XmlContentType;
                        request.Credentials = new NetworkCredential(userName, password);

                        using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                        {
                            data = ReadResponse(response);

                            if (response.StatusCode != HttpStatusCode.OK)
                            {
                                MessageBox.Show(string.Format("Unexpected status code returned (HttpWebResponse): {0}", response.StatusCode), title);
                                return;
                            }
                        }
                    }
                    catch (WebException ex)
                    {
                        using (HttpWebResponse response = ex.Response as HttpWebResponse)
                        {
                            MessageBox.Show(string.Format("Unexpected status code returned (WebException): {0}", response.StatusCode), title);
                            return;
                        }
                    }
                    
                    // Changed to LINQ to XML XElement processing for consistency with create code
                    // Sample code used XML DOM (XmlDocument) syntax as shown below
                    string updatedData = null;
                    bool useXElement = true;
                    if (useXElement)
                    {
                        // XDocument processing
                        XNamespace xsi = "http://www.w3.org/2001/XMLSchema-instance";
                        XElement orderXml = XElement.Parse(data);
                        XElement ts = orderXml.Element("TimeStamp");
                        if ( ts == null)
                        {
                            // Add the TimeStamp element
                            XElement lastXe = orderXml.Element("ShipCountry");
                            if (lastXe != null)
                            {
                                XElement newXe = new XElement("TimeStamp",
                                    new XAttribute(xsi + "type", "x:dateTime"),
                                    DateTime.UtcNow.ToString("s"));
                                lastXe.AddAfterSelf(newXe);
                            }
                        }
                        else
                        {
                            // Remove the TimeStamp element
                             ts.Remove();
                        }
                        updatedData = orderXml.ToString(SaveOptions.DisableFormatting);
                    }
                    else
                    {
                        // Original XmlDocument processing
                        XmlDocument orderXml = new XmlDocument();
                        data = FormatXml(data);
                        orderXml.LoadXml(data);
                        XmlElement ts = orderXml.GetElementById("TimeStamp");
                        if (ts == null)
                        {
                            // Add new flex property "TimeStamp" to the entity
                            XmlElement newElem = orderXml.CreateElement("TimeStamp");
                            newElem.InnerXml = DateTime.UtcNow.ToString("s");
                            newElem.SetAttribute("type", "http://www.w3.org/2001/XMLSchema-instance", "x:dateTime");
                            orderXml.DocumentElement.AppendChild(newElem);
                        }
                        else
                        {
                            // Remove the TimeStamp
                            orderXml.RemoveChild(ts);
                        }
                        // Send the entire entity for the update
                        updatedData = orderXml.OuterXml;
                    }

                    try
                    {
                        HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(entityUri);
                        request.Method = HttpPutMethod;
                        UTF8Encoding encoding = new UTF8Encoding();
                        request.ContentLength = encoding.GetByteCount(updatedData);
                        request.ContentType = XmlContentType;
                        request.Credentials = new NetworkCredential(userName, password);

                        // Write the request data over the wire.
                        using (Stream reqStm = request.GetRequestStream())
                        {
                            reqStm.Write(encoding.GetBytes(updatedData), 0,
                                         encoding.GetByteCount(updatedData));
                        }

                        // Get the response and read it in to a string.
                        using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                        {
                            if (response.StatusCode != HttpStatusCode.OK)
                            {
                                MessageBox.Show(string.Format("Unexpected status code returned (HttpWebResponse): {0}", response.StatusCode), title);
                                return;
                            }
                        }
                    }
                    catch (WebException ex)
                    {
                        using (HttpWebResponse response = ex.Response as HttpWebResponse)
                        {
                            MessageBox.Show(string.Format("Unexpected status code returned (Web Exception): {0}", response.StatusCode), title);
                            return;
                        }
                    }
                    entities +=1;
                }
            }
            else
            {
                using (SitkaSoapServiceClient proxy = new SitkaSoapServiceClient())
                {
                    // Supply credentials
                    proxy.ClientCredentials.UserName.UserName = userName;
                    proxy.ClientCredentials.UserName.Password = password;

                    StringBuilder payload = new StringBuilder();
                    sw.Start();
                    if (updateOrders)
                    {
                        foreach (Entity order in brazilOrders)
                        {
                            // Set scope to a specific entity
                            Scope entityScope = new Scope();
                            entityScope.AuthorityId = authorityId;
                            entityScope.ContainerId = containerId;
                            entityScope.EntityId = order.Id;

                            // If TimeStamp field is present, remove it otherwise add it
                            if (order.Properties.Keys.Contains("TimeStamp"))
                                order.Properties.Remove("TimeStamp");
                            else
                                order.Properties.Add("TimeStamp", DateTime.UtcNow);

                            // Update the order
                            proxy.Update(entityScope, order);
                            action = "updated";
                        }
                    }
                    else
                    {
                        foreach (Entity order in brazilOrders)
                        {
                            // Set scope to a container
                            Scope containerScope = new Scope();
                            containerScope.AuthorityId = authorityId;
                            containerScope.ContainerId = containerId;

                            // Add the order
                            proxy.Create(containerScope, order);
                            payload.Append("Id = " + order.Id + "\r\n");
                            payload.Append("Kind = " + order.Kind + "\r\n");
                            foreach (KeyValuePair<string, object> kvp in order.Properties)
                                payload.Append(kvp.Key + " = " + kvp.Value.ToString() + "\r\n");
                            payload.Append("\r\n");
                            action = "added";
                        }
                    }
                    entities = brazilOrders.Count();
                }
            }
            sw.Stop();
            double time = sw.ElapsedMilliseconds / 1000d;
            txtPayload.Text = entities.ToString() +
                " Brazilian orders " + action + " in " +
                time.ToString("0.000") + " sec. = " +
                (entities / time).ToString("#0.000") +
                " orders/sec.\r\n\r\n";
            txtEntitySet.Text = "Update Orders";
            txtEntities.Text = entities.ToString();
            txtTimeUpload.Text = time.ToString();
            txtTimeXml.Text = "0.000";
            AddTimingData();
            this.Cursor = Cursors.Default;
        }
        
        // Delete all Brazilian orders 
        private void btnDeleteOrders_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            if (chkUseREST.Checked)
            {

            }
            else
            {
                using (SitkaSoapServiceClient proxy = new SitkaSoapServiceClient())
                {
                    // Supply credentials
                    proxy.ClientCredentials.UserName.UserName = userName;
                    proxy.ClientCredentials.UserName.Password = password;

                    Stopwatch sw = new Stopwatch();
                    sw.Start();
                    foreach (Entity order in brazilOrders)
                    {
                        // Set scope to a specific entity
                        Scope entityScope = new Scope();
                        entityScope.AuthorityId = authorityId;
                        entityScope.ContainerId = containerId;
                        entityScope.EntityId = order.Id;
                        proxy.Delete(entityScope);
                    }
                    sw.Stop();
                    double time = (sw.ElapsedMilliseconds / 1000d);
                    txtPayload.Text = brazilOrders.Count().ToString() +
                        " Brazilian orders deleted in " +
                        time.ToString("0.000") + " sec. = " +
                        (brazilOrders.Count() / time).ToString("#0.000") +
                        " orders/sec.\r\n\r\n";
                    txtEntitySet.Text = "Delete Orders";
                    txtEntities.Text = brazilOrders.Count().ToString();
                    txtTimeUpload.Text = time.ToString();
                    txtTimeXml.Text = "0.000";
                    AddTimingData();
                }
            }
            this.Cursor = Cursors.Default;
            // Image of deleted orders is still present
            string msg = "Do you want to add back the orders you just deleted?";
            if (MessageBox.Show(msg, title, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                UpdateOrAddBrazilianOrders(false);
        }

        private void chkUseREST_CheckedChanged(object sender, EventArgs e)
        {
            // Must get orders of each type before updating/deleting
            btnUpdateOrders.Enabled = false;
            btnDeleteOrders.Enabled = false;
        }

        // Startup splashscreen
        private void ShowSplashscreen(Form startup)
        {
            string msgStartup = "Connecting to SQL Server Data Services (SSDS Sprint 3) " +
                " and obtaining the list of Containers and Entities in the '" +
                txtAuthority.Text + "' Authority.";
            startup.Text = title;
            startup.FormBorderStyle = FormBorderStyle.FixedSingle;
            startup.MaximizeBox = false;
            startup.MinimizeBox = false;
            startup.Width = 300;
            startup.Height = 120;
            startup.Font = new System.Drawing.Font("Calibri", 11F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            startup.StartPosition = FormStartPosition.CenterScreen;
            TextBox txtStartup = new TextBox();
            txtStartup.Location = new Point(10, 10);
            txtStartup.Multiline = true;
            txtStartup.Width = 280;
            txtStartup.Height = 80;
            txtStartup.TabStop = false;
            txtStartup.BorderStyle = BorderStyle.None;
            txtStartup.BackColor = System.Drawing.SystemColors.Control;
            txtStartup.Text = msgStartup;
            startup.Controls.Add(txtStartup);
            startup.UseWaitCursor = true;
            startup.Show();
            Application.DoEvents();
        }
        
        // ********************************************************************************
        // The following REST procedures are from the sample code with slight modifications
        // ********************************************************************************
        
        public string ProcessEntity(string containerUri, string requestPayload, string httpMethod)
        {
            // Create a single entity instance
            string entityUri = null;

            if (String.IsNullOrEmpty(containerUri))
            {
                throw new ArgumentOutOfRangeException(containerUri);
            }

            if (String.IsNullOrEmpty(requestPayload))
            {
                throw new ArgumentOutOfRangeException(requestPayload);
            }

            try
            {
                // Create the request to send.
                WebRequest request = HttpWebRequest.Create(containerUri);
                request.Credentials = new NetworkCredential(userName, password);
                // POST=create; PUT=update; DELETE=delete; GET=retrieve
                request.Method = httpMethod;
                UTF8Encoding encoding = new UTF8Encoding();
                request.ContentLength = encoding.GetByteCount(requestPayload);
                request.ContentType = "application/xml";
                // Accumulate message size
                contentBytes += request.ContentLength;

                // Write the request data over the wire.
                using (Stream reqStm = request.GetRequestStream())
                {
                    reqStm.Write(encoding.GetBytes(requestPayload), 0,
                                 encoding.GetByteCount(requestPayload));
                }

                // Get the response and read it in to a string.
                using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                {

                    if (response.StatusCode != HttpStatusCode.Created)
                    {
                        string msg = String.Format("Unexpected status code returned, entity not created: {0}", response.StatusCode) +
                            "\r\n\r\nDo you want to continue?";
                        if (MessageBox.Show(msg, title, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                        {
                            return "exit";
                        };
                    }
                }
            }
            catch (WebException ex)
            {
                string msg = null;
                if (ex.Response != null)
                    msg = ex.Message;
                else
                {
                    using (HttpWebResponse response = ex.Response as HttpWebResponse)
                    {
                        msg = String.Format("Unexpected status code returned (WebException): {0}", response.StatusCode) +
                            "\r\n\r\nDo you want to continue?";
                    }
                }
                if (MessageBox.Show(msg, title, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                {
                    return "exit";
                };
            }
            return entityUri;
        }

        private string CreateContainer(string authorityUri)
        {
            if (String.IsNullOrEmpty(authorityUri))
            {
                throw new ArgumentOutOfRangeException("authorityUri");
            }
            string containerUri = null;
            try
            {
                // We need to create XML Payload for the 
                // new container request.
                const string ContainerTemplate =
                    @"<s:Container xmlns:s='http://schemas.microsoft.com/sitka/2008/03/'>
                        <s:Id>{0}</s:Id>
                      </s:Container>";

                // Data going over to the server
                containerId = txtContainer.Text;
                string requestPayload = string.Format(ContainerTemplate, containerId);

                HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(authorityUri);
                // Security requirement - need to pass userid, password
                request.Credentials = new NetworkCredential(userName, password);

                // POST=create; PUT=update; DELETE=delete; GET=retrieve
                request.Method = HttpPostMethod;
                UTF8Encoding encoding = new UTF8Encoding();
                request.ContentLength = encoding.GetByteCount(requestPayload);
                request.ContentType = XmlContentType;

                // Write the request data over the wire (1st container).
                using (Stream reqStm = request.GetRequestStream())
                {
                    reqStm.Write(encoding.GetBytes(requestPayload), 0,
                                 encoding.GetByteCount(requestPayload));
                }

                // Get the response and read it in to a string.
                using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                {
                    if (response.StatusCode != HttpStatusCode.Created)
                    {
                        MessageBox.Show("Failed to create container");
                    }
                    // Container created successfully. Get the container URI.
                    // HttpResponseHeader.Location = null in current SSDS Spring
                    // containerUri = response.Headers[HttpResponseHeader.Location];
                    // So create containerUri as elsewhere
                    containerUri = authorityUri + txtContainer.Text;
                    return containerUri;
                }
            }
            catch (WebException ex)
            {
                using (HttpWebResponse response = ex.Response as HttpWebResponse)
                {
                    if (response != null)
                    {
                        string errorMsg = ReadResponse(response);
                        if (errorMsg.Contains("already exists"))
                        {
                            if (chkContainerPerEntity.Checked)
                            {
                                string msg = "A container for each EntitySet already exists. " +
                                "Do you want to use the existing containers?" +
                                "\r\n\r\nClick No to create a new container for each EntitySet." + 
                                "\r\n\r\nClick Cancel to delete the existing containers.";
                                DialogResult result = MessageBox.Show(msg, title, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
                                if (result == DialogResult.Yes)
                                {
                                    // Turn off automatic processing
                                    chkCustomers.Checked = false;
                                    chkOrders.Checked = false;
                                    chkEmployees.Checked = false;
                                    chkShippers.Checked = false;
                                    chkOrderDetails.Checked = false;
                                    chkProducts.Checked = false;
                                    chkCategories.Checked = false;
                                    chkSuppliers.Checked = false;
                                    return authorityUri + containerId;
                                }
                                else if (result == DialogResult.No)
                                {
                                    txtPayload.Text = "Deleted the following containers:\r\n";
                                    string[] containers = { "Customers", "Orders", "Employees", "Shippers", "OrderDetails", "Products", "Categories", "Suppliers" };
                                    for (int i = 0; i < 8; i++)
                                    {
                                        containerId = containers[i];
                                        if (DeleteContainer(authorityUri))
                                            txtPayload.Text += "   " + containers[i] + " container deleted \r\n";
                                        else
                                            txtPayload.Text += "   " + containers[i] + " container not found \r\n";
                                    }
                                    txtContainer.Text = containers[0];
                                    containerId = txtContainer.Text;
                                    CreateContainer(authorityUri);
                                    MessageBox.Show("Click OK to continue", title);
                                    return authorityUri + containers[0];
                                }
                                else
                                {
                                    txtPayload.Text = "Deleted the following containers:\r\n";
                                    string[] containers = { "Customers", "Orders", "Employees", "Shippers", "OrderDetails", "Products", "Categories", "Suppliers" };
                                    for (int i = 0; i < 8; i++)
                                    {
                                        containerId = containers[i];
                                        if (DeleteContainer(authorityUri))
                                            txtPayload.Text += "   " + containers[i] + " container deleted \r\n";
                                        else
                                            txtPayload.Text += "   " + containers[i] + " container not found \r\n";
                                    }
                                    txtContainer.Text = containers[0];
                                    containerId = txtContainer.Text;
                                    return "deleted";
                                }
                            }
                            else
                            {
                                string msg = "Container '" + containerId + "' already exists. " +
                                    "Do you want to use the existing container?" +
                                        "\r\n\r\nClick No to replace the existing container with a new empty container." +
                                        "\r\n\r\nClick Cancel to delete the existing container.";
                                DialogResult result = MessageBox.Show(msg, title, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
                                if (result == DialogResult.Yes)
                                {
                                    return authorityUri + containerId;
                                }
                                else if (result == DialogResult.No)
                                {
                                    DeleteContainer(authorityUri);
                                    CreateContainer(authorityUri);
                                    return authorityUri + containerId;
                                }
                                else
                                {
                                    // Canceled
                                    containerId = txtContainer.Text;
                                    DeleteContainer(authorityUri);
                                    txtPayload.Text += "\r\n" + containerId + " container deleted";
                                    return "deleted";
                                }
                            }
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error: {0}", errorMsg) +
                            string.Format("\r\nUnexpected status code returned: {0} ", response.StatusCode));
                        }
                    }
                }
            }
            return containerUri;
        }

        public bool DeleteContainer(string authorityUri)
        {
            // Delete the specified container
            try
            {
                string containerUri = authorityUri + containerId;
                WebRequest request = HttpWebRequest.Create(containerUri);
                request.Credentials = new NetworkCredential(userName, password);
                request.Method = HttpDeleteMethod;
                request.ContentLength = 0;

                // Get the response and read it in to a string.
                using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                {
                    if (response.StatusCode != HttpStatusCode.OK)
                    {
                        MessageBox.Show("Container deletion failed");
                    }
                    return true;
                }
            }
            catch (WebException ex)
            {
                return false;
            }
        }

        public static string ReadResponse(HttpWebResponse response)
        {
            // Begin by validating our inbound parameters.
            //
            if (response == null)
            {
                throw new ArgumentNullException("response", "Value cannot be null");
            }

            // Then, open up a reader to the response and read the contents to a string
            // and return that to the caller.
            //
            string responseBody = "";
            using (Stream rspStm = response.GetResponseStream())
            {
                using (StreamReader reader = new StreamReader(rspStm))
                {
                    responseBody = reader.ReadToEnd();
                }
            }

            return responseBody;
        }

    }
}
